unit UFormMain;

{$mode delphi}{$H+}

interface

uses
  Classes, SysUtils, Forms, Controls, Graphics, Dialogs, StdCtrls, FileUtil;

type

  { TFormMain }

  TFormMain = class(TForm)
    ButtonProcess: TButton;
    EditSrcDir: TEdit;
    EditDstDir: TEdit;
    Label1: TLabel;
    Label2: TLabel;
    MemoLog: TMemo;
    procedure ButtonProcessClick(Sender: TObject);
    procedure FormActivate(Sender: TObject);
  private
    Initialized: Boolean;
    procedure FormatCodePas(FileName: string);
    procedure FormatCodeLfm(FileName: string);
    function IsAlpha(C: Char): Boolean;
    function ReplaceText(Text: string; FromText, ToText: string): string;
    function ReplaceIdent(Text: string; FromText, ToText: string): string;
  public

  end;

var
  FormMain: TFormMain;
  TypeNames: array[0..15] of string = ('Bitmap', 'Form', 'Font', 'WinControl', 'ListBox',
    'GraphicControl', 'ScrollBar', 'Canvas', 'RasterImage', 'PortableNetworkGraphic',
    'JpegImage', 'Control', 'MenuItem', 'PopupMenu', 'ControlBorderSpacing', 'Edit');

implementation

{$R *.lfm}

type
  TFilterMethod = function (FileName: string): Boolean of object;
  TFileNameMethod = procedure (FileName: string) of object;

procedure SearchFiles(List: TStrings; Dir: string;
  FilterMethod: TFilterMethod = nil; FileNameMethod: TFileNameMethod = nil);
var
  SR: TSearchRec;
  FullName: string;
begin
  List.BeginUpdate;
  try
    Dir := IncludeTrailingPathDelimiter(Dir);
    if FindFirst(Dir + '*', faAnyFile, SR) = 0 then begin
      try
        repeat
          if (SR.Name = '.') or (SR.Name = '..') then Continue;
          FullName := Dir + SR.Name;
          if not Assigned(FilterMethod) or (Assigned(FilterMethod) and FilterMethod(FullName)) then begin;
            if Assigned(FileNameMethod) then
              FileNameMethod(Dir + SR.Name);
            List.Add(FullName);
          end;
          if (SR.Attr and faDirectory) <> 0 then
            SearchFiles(List, FullName, FilterMethod, FileNameMethod);
        until FindNext(SR) <> 0;
      finally
        FindClose(SR);
      end;
    end;
  finally
    List.EndUpdate;
  end;
end;

{ TFormMain }

function TFormMain.IsAlpha(C: Char): Boolean;
begin
  Result := (C in ['a'..'z']) or (C in ['A'..'Z']) or (C = '_');
end;

procedure TFormMain.FormActivate(Sender: TObject);
begin
  if not Initialized then begin
    Initialized := True;
  end;
end;

procedure TFormMain.ButtonProcessClick(Sender: TObject);
var
  Files: TStringList;
  I: Integer;
  FileNameSrc: string;
  FileNameDst: string;
  DirSrc: string;
  DirDst: string;
begin
  DirSrc := EditSrcDir.Text;
  DirDst := EditDstDir.Text;

  MemoLog.Lines.BeginUpdate;
  MemoLog.Lines.Clear;
  Files := TStringList.Create;
  SearchFiles(Files, DirSrc, nil, nil);
  for I := 0 to Files.Count - 1 do begin
    MemoLog.Lines.Add(Files[I]);
    FileNameSrc := Files[I];
    if DirectoryExists(FileNameSrc) then continue;
    if FileExists(FileNameSrc) then begin
      FileNameDst := DirDst + Copy(FileNameSrc, Length(DirSrc) + 1, MaxInt);
      ForceDirectories(ExtractFileDir(FileNameDst));
      CopyFile(FileNameSrc, FileNameDst);

      if (ExtractFileExt(FileNameSrc) = '.pas') or (ExtractFileExt(FileNameSrc) = '.lpr') then
        FormatCodePas(FileNameDst);
      if ExtractFileExt(FileNameSrc) = '.lfm' then
        FormatCodeLfm(FileNameDst);
    end;
  end;
  Files.Free;
  MemoLog.Lines.EndUpdate;
end;

procedure TFormMain.FormatCodePas(FileName: string);
var
  Lines: TStringList;
  I, J: Integer;
  Line: string;
  Differences: Integer;
  UsesSection: Boolean;
begin
  Lines := TStringList.Create;
  try
    Lines.LoadFromFile(FileName);
    Differences := 0;
    for I := 0 to Lines.Count - 1 do begin
      Line := Lines[I];
      for J := 0 to Length(TypeNames) - 1 do
        Line := ReplaceIdent(Line, 'T' + TypeNames[J], 'TDpi' + TypeNames[J]);
      Line := ReplaceIdent(Line, 'Screen', 'DpiScreen');
      Line := ReplaceIdent(Line, 'Mouse', 'DpiMouse');
      Line := ReplaceIdent(Line, 'Application', 'DpiApplication');
      Line := StringReplace(Line, 'BitBlt(', 'DpiBitBlt(', [rfReplaceAll]);
      Line := StringReplace(Line, 'BitBltCanvas(', 'DpiBitCanvas(', [rfReplaceAll]);
      Line := StringReplace(Line, 'GetSystemMetrics(', 'DpiGetSystemMetrics(', [rfReplaceAll]);
      Line := StringReplace(Line, 'CreateRectRgn(', 'DpiCreateRectRgn(', [rfReplaceAll]);
      Line := StringReplace(Line, 'ScrollDC(', 'DpiScrollDC(', [rfReplaceAll]);
      if Lines[I] <> Line then Inc(Differences);
      Lines[I] := Line;
    end;

    // Add UDpiControls to uses clause
    if Differences > 0 then begin
      UsesSection := False;
      for I := 0 to Lines.Count - 1 do begin
        if UsesSection then begin
          Lines[I] := '  UDpiControls, ' + TrimLeft(Lines[I]);
          Break;
        end;
        if Lines[I] = 'uses' then UsesSection := True;
      end;
    end;
    Lines.SaveToFile(FileName);
  finally
    Lines.Free;
  end;
end;

procedure TFormMain.FormatCodeLfm(FileName: string);
var
  Lines: TStringList;
  I, J: Integer;
  Line: string;
begin
  Lines := TStringList.Create;
  try
    Lines.LoadFromFile(FileName);
    for I := 0 to Lines.Count - 1 do begin
      Line := Lines[I];
      for J := 0 to Length(TypeNames) - 1 do
        Line := StringReplace(Line, ': T' + TypeNames[J], ': TDpi' + TypeNames[J], [rfReplaceAll]);
      Lines[I] := Line;
    end;
    Lines.SaveToFile(FileName);
  finally
    Lines.Free;
  end;
end;

function TFormMain.ReplaceText(Text: string; FromText, ToText: string): string;
begin
  Result := StringReplace(Text, FromText + '.', ToText + '.', [rfReplaceAll]);
  Result := StringReplace(Result, FromText + ';', ToText + ';', [rfReplaceAll]);
  Result := StringReplace(Result, FromText + ')', ToText + ')', [rfReplaceAll]);
  Result := StringReplace(Result, FromText + ',', ToText + ',', [rfReplaceAll]);
  Result := StringReplace(Result, FromText + ' ', ToText + ' ', [rfReplaceAll]);
  Result := StringReplace(Result, '(' + FromText + ')', '(' + ToText + ')', [rfReplaceAll]);
end;

function TFormMain.ReplaceIdent(Text: string; FromText, ToText: string): string;
type
  TState = (stNone, stIdent);
var
  I: Integer;
  State: TState;
  Ident: string;
  IdentStart: Integer;
begin
  I := 1;
  State := stNone;
  IdentStart := 0;
  while I <= Length(Text) do begin
    if State = stNone then begin
      if IsAlpha(Text[I]) then begin
        State := stIdent;
        Ident := Text[I];
        IdentStart := I;
      end;
    end else
    if State = stIdent then begin
      if IsAlpha(Text[I]) then begin
        Ident := Ident + Text[I];
      end else begin
        if Ident = FromText then begin
          Text := Copy(Text, 1, IdentStart - 1) + ToText + Copy(Text, I, MaxInt);
        end;
        State := stNone;
      end;
    end;
    Inc(I);
  end;
  Result := Text;
end;


end.

